﻿using System.Text.Json;

namespace SetlistBuilder
{
    internal class MultiPageBuilder
    {
        private readonly string _manifestFilename;
        private readonly string _inputPath;
        private readonly IOrderedEnumerable<Tab> _index;
        internal MultiPageBuilder(string manifestFilename, string inputPath)
        {
            _manifestFilename = manifestFilename;
            _inputPath = inputPath;
            _index = JsonSerializer.Deserialize<List<Tab>>(File.ReadAllText($"{_inputPath}\\{_manifestFilename}"))?.OrderBy(tab => tab.Order) ?? throw new Exception("unable to open or deserialize manifest");
        }

        internal void Build(string outputPath)
        {
            if(Directory.Exists(outputPath)) Directory.Delete(outputPath, true);
            Directory.CreateDirectory(outputPath);

            MakeIndex(outputPath);

            Tab? prev = null;
            Tab? current = null;

            foreach (var next in _index)
            {
                if (current != null)
                    BuildPage(_inputPath, current, outputPath, prev ?? current, next);
                prev = current;
                current = next;
            }

            BuildPage(_inputPath, current, outputPath, (prev ?? current), current);
        }

        private void BuildPage(string inputFolder, Tab current, string outputFolder, Tab previous, Tab next)
        {     
            var inputPath = $"{inputFolder}\\{current.File}";
            var outputPath = $"{outputFolder}\\{current.Title} - {current.Artist}.html";

            using (var output = new StreamWriter(new FileStream(outputPath, FileMode.OpenOrCreate)))
            using (var input = new StreamReader(inputPath))
            {
                var prev = $"{previous.Title} - {previous.Artist}.html";
                var title = $"{current.Title} - {current.Artist}";
                output.Write($"<html><head></head><body><div><a href=\"{prev}\">Previous - {prev}</a><h4>{title}</h4><pre>");
                output.Write(input.ReadToEnd());
                var nextTab = $"{next.Title} - {next.Artist}.html";
                output.Write($"</pre></pre><a href=\"index.html\">Index</a> <a href=\"{nextTab}\">Next - {nextTab}</a></div></body></html>");
                output.Flush();
                output.Close();
            }
        }

        private void MakeIndex(string outputFolder)
        {
            var outputPath = $"{outputFolder}\\index.html";
            using (var output = new StreamWriter(new FileStream(outputPath, FileMode.OpenOrCreate)))
            {
                output.Write("<html><head></head><body>");
                output.Write("<h1 id=\"index\">Index</h1><ol>");
                foreach (var tab in _index)
                {
                    var tabId = $"{tab.Title} - {tab.Artist}";
                    output.Write($"<li><a href=\"{tabId}.html\">{tabId}</a></li>");
                }
                output.Write("</ol></body></html>");
                output.Flush();
                output.Close();
            }
        }
    }
}
