﻿namespace SetlistBuilder
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var options = new Queue<string>(args);
            Modes _mode = Modes.SinglePage;
            string inputPath = null;
            string outputPath = null;

            while (options.TryDequeue(out string option))
            {
                switch (option)
                {
                    case "-s":
                        _mode = Modes.SinglePage;
                        break;
                    case "-m":
                        _mode = Modes.MultiPage;
                        break;
                    case "-e":
                        _mode = Modes.EPub;
                        break;
                    case "-i":
                        if (options.TryDequeue(out inputPath) == false)
                            throw new Exception("No input path provided");
                        if (!Directory.Exists(inputPath))
                            throw new Exception($"Input path {inputPath} is not a valid path");
                        break;
                    case "-o":
                        if (options.TryDequeue(out outputPath) == false)
                            throw new Exception("No output path provided");
                        break;
                    default:
                        throw new Exception($"Invalid option {option}");
                }
            }

            if (inputPath == null) throw new Exception("Input path not provided");

            switch (_mode)
            {
                case Modes.SinglePage:
                    new SinglePageBuilder("manifest.json", inputPath).Build();
                    break;
                case Modes.MultiPage:
                    new MultiPageBuilder("manifest.json", inputPath).Build(outputPath ?? throw new Exception("outputPath not provided"));
                    break;
                case Modes.EPub:
                    throw new Exception("incomplete feature, please do not use yet");
            }
        }
    }

    enum Modes
    {
        SinglePage,
        MultiPage,
        EPub
    }
}