﻿using System.Runtime.CompilerServices;
using System.Text.Json;

namespace SetlistBuilder
{
    class SinglePageBuilder
    {
        private readonly string _manifestFilename;
        private readonly string _inputPath;
        private readonly IOrderedEnumerable<Tab> _index;
        internal SinglePageBuilder(string manifestFilename, string inputPath)
        {
            _manifestFilename = manifestFilename;
            _inputPath = inputPath;
            _index = JsonSerializer.Deserialize<List<Tab>>(File.ReadAllText($"{_inputPath}\\{_manifestFilename}"))?.OrderBy(tab => tab.Order) ?? throw new Exception("unable to open or deserialize manifest");
        }

        internal void Build()
        {
            File.Delete($"{_inputPath}\\Printable Tabs.html");

            using (var output = new StreamWriter(new FileStream($"{_inputPath}\\Printable Tabs.html", FileMode.OpenOrCreate)))
            {
                output.Write("<html><head><style>@media print {.page-break { break-after: page; }}</style><body>");
                MakeIndex(output);
                foreach (var tab in _index)
                {
                    using (var input = new StreamReader($"{_inputPath}\\{tab.File}"))
                    {
                        var tabId = $"{tab.Title} - {tab.Artist}";
                        output.Write($"<div class=\"page-break\" id=\"{tabId}\"><h4>{tabId}</h4><pre>");
                        output.Write(input.ReadToEnd());
                        output.Write($"</pre><a href=\"#index\">Back to Index</a></div>");
                    }
                }
                output.Write("</body></html>");
                output.Flush();
                output.Close();
            }
        }

        private void MakeIndex(StreamWriter output)
        {
            output.Write("<h1 id=\"index\">Index</h1><ol>");
            foreach (var tab in _index)
            {
                var tabId = $"{tab.Title} - {tab.Artist}";
                output.Write($"<li><a href=\"#{tabId}\">{tabId}</a></li>");
            }
            output.Write("</ol>");
        }
    }
}
