﻿using QuickEPUB;
using System.Text.Json;

namespace SetlistBuilder
{
    internal class EpubBuilder
    {
        private readonly IOrderedEnumerable<Tab> _index;
        private readonly string _manifestFilename;
        private readonly string _inputPath;

        internal EpubBuilder(string manifestFilename, string inputPath)
        {
            _manifestFilename = manifestFilename;
            _inputPath = inputPath;
            _index = JsonSerializer.Deserialize<List<Tab>>(File.ReadAllText($"{_inputPath}\\{_manifestFilename}"))?.OrderBy(tab => tab.Order) ?? throw new Exception("unable to open or deserialize manifest");
        }

        internal void Build()
        {
            File.Delete($"{_inputPath}\\tabs.epub");
            var epub = new Epub("Tabs", "Barry and the Hatchets");

            foreach (var tab in _index)
                using (var input = new StreamReader($"{_inputPath}\\{tab.File}"))
                    epub.AddSection($"{tab.Title} - {tab.Artist}", input.ReadToEnd());

            using (var writestream = new FileStream($"{_inputPath}\\tabs.epub", FileMode.Create))
                epub.Export(writestream);
        }
    }
}
